import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.Statement;
import java.sql.Types;
import java.util.Vector;

import quadbase.reportdesigner.util.IParameterizedDataSource;
import quadbase.reportdesigner.util.IQueryInParam;
import quadbase.reportdesigner.util.IQueryMultiValueInParam;
import quadbase.reportdesigner.util.IQueryParamValuesProvider;
import quadbase.reportdesigner.util.IResultSet;
import quadbase.reportdesigner.util.QueryResultSet;
import quadbase.reportdesigner.util.SimpleQueryMultiValueInParam;




public class ClassDataMapToColumn implements IParameterizedDataSource {

	public IQueryInParam[] getParameters() {

		mySimpleQueryMultiValueInParam[] params = new mySimpleQueryMultiValueInParam[1];
		params[0] = new mySimpleQueryMultiValueInParam("region", "Select Region(s):", true, "Customers", "Region", Types.VARCHAR, "East", null);

		return params;

	}

	// Getting the result set for a query with a multivalue parameter
	public IResultSet getResultSet(IQueryInParam[] params) {

		QueryResultSet data = null;
		ResultSet rs = null;
		// Default param value
		String paramValue = "'East'";

		if ((params != null) && (params.length >= 1)) {
			Vector selectedValues = null;
			
			// First extract all selected values from the parameter
			if (params[0] instanceof IQueryMultiValueInParam)
				selectedValues = ((IQueryMultiValueInParam)params[0]).getValues();

			// Concatenate each selected value into a single string
			for (int i = 0; i < selectedValues.size(); i++) {
				if ((selectedValues.get(i) != null) && (selectedValues.get(i) instanceof String)) {
					if (i == 0) paramValue = "'" + (String)selectedValues.get(i) + "'";
					else paramValue += ",'" + (String)selectedValues.get(i) + "'";
				}
			}
		}

		// Add the string to the query within the IN clause
		String myQuery = "select cu.region, c.categoryname, count(o.orderid), sum(od.quantity), sum(p.unitprice * od.quantity) " +
				"from customers cu, categories c, products p, orders o, order_details od " +
				"where cu.customerid = o.customerid and c.categoryid = p.categoryid and " +
				"p.productid = od.productid and o.orderid = od.orderid " +
				"and cu.region in (" + paramValue + ") " +
				"group by cu.region, c.categoryname";

		// Execute the query and return the result set
		try {
			Class.forName("org.hsqldb.jdbcDriver");
			String url = "jdbc:hsqldb:help/examples/DataSources/database/woodview";
			Connection conn = DriverManager.getConnection(url, "sa", "");
			Statement stmt = conn.createStatement();

			rs = stmt.executeQuery(myQuery);
			data = new QueryResultSet(rs);
//			conn.close();

		} catch (Exception ex) {
			ex.printStackTrace();
		}
		return data;
	}

	public class mySimpleQueryMultiValueInParam extends SimpleQueryMultiValueInParam implements IQueryParamValuesProvider {
		public String paramName, promptName, tableName, colName;
		boolean mapToColumn;
		int sqlType;
		Object defaultValue;
		Vector values;

		public mySimpleQueryMultiValueInParam(String paramName, String promptName, boolean mapToColumn, String tableName, String colName, int sqlType, Object defaultValue, Vector values) {
			super(paramName, promptName, mapToColumn, tableName, colName, sqlType, defaultValue, values);
		}

		public Vector getSelectionChoices() {
			System.out.println("getSelectionChoices called");
			try {
				Class.forName("org.hsqldb.jdbcDriver");

				String url = "jdbc:hsqldb:help/examples/DataSources/database/woodview";
				Connection conn = DriverManager.getConnection(url, "sa", "");
				Statement stmt = conn.createStatement();
				String query = "SELECT DISTINCT " + getColumnName() + " FROM " + getTableName();
				ResultSet rs = stmt.executeQuery(query);
				Vector v = new Vector();

				while (rs.next()) {
					switch (getSqlType()) {
					case Types.INTEGER:
						v.add(new Integer(rs.getInt(1)));
						break;

					case Types.VARCHAR:
						v.add(rs.getString(1));
						break;
					}
				}

				stmt.close();
				conn.close();

				return v;
			} catch (Exception ex) {
				ex.printStackTrace();
			}
			return null;
		}
	}
}